@@ -253,15 +253,17 @@ class VectorQuantize(nn.Module):
         n_embed = None,
         codebook_dim = None,
         decay = 0.8,
-        orthogonal_reg_weight = 0.,
-        commitment_weight = None,
         eps = 1e-5,
         kmeans_init = False,
         kmeans_iters = 10,
         use_cosine_sim = False,
         threshold_ema_dead_code = 0,
         channel_last = True,
-        commitment = 1. # deprecate in next version, turn off by default
+        accept_image_fmap = False,
+        commitment_weight = None,
+        commitment = 1., # deprecate in next version, turn off by default
+        orthogonal_reg_weight = 0.,
+        orthogonal_reg_active_codes_only = False
     ):
         super().__init__()
         n_embed = default(n_embed, codebook_size)
@@ -275,7 +277,9 @@ class VectorQuantize(nn.Module):
 
         self.eps = eps
         self.commitment_weight = default(commitment_weight, commitment)
+
         self.orthogonal_reg_weight = orthogonal_reg_weight
+        self.orthogonal_reg_active_codes_only = orthogonal_reg_active_codes_only
 
         codebook_class = EuclideanCodebook if not use_cosine_sim \
                          else CosineSimCodebook
@@ -291,6 +295,8 @@ class VectorQuantize(nn.Module):
         )
 
         self.codebook_size = codebook_size
+
+        self.accept_image_fmap = accept_image_fmap
         self.channel_last = channel_last
 
     @property
@@ -298,12 +304,16 @@ class VectorQuantize(nn.Module):
         return self._codebook.embed
 
     def forward(self, x):
-        device, codebook_size = x.device, self.codebook_size
+        shape, device, codebook_size = x.shape, x.device, self.codebook_size
 
-        need_transpose = not self.channel_last
+        need_transpose = not self.channel_last and not self.accept_image_fmap
+
+        if self.accept_image_fmap:
+            height, width = x.shape[-2:]
+            x = rearrange(x, 'b c h w -> b (h w) c')
 
         if need_transpose:
-            x = rearrange(x, 'b n d -> b d n')
+            x = rearrange(x, 'b d n -> b n d')
 
         x = self.project_in(x)
 
@@ -320,12 +330,25 @@ class VectorQuantize(nn.Module):
                 loss = loss + commit_loss * self.commitment_weight
 
             if self.orthogonal_reg_weight > 0:
+                codebook = self.codebook
+
+                if self.orthogonal_reg_active_codes_only:
+                    # only calculate orthogonal loss for the activated codes for this batch
+                    unique_code_ids = torch.unique(embed_ind)
+                    codebook = self.codebook[unique_code_ids]
+                else:
+                    codebook = self.codebook
+
                 orthogonal_reg_loss = orthgonal_loss_fn(self.codebook)
                 loss = loss + orthogonal_reg_loss * self.orthogonal_reg_weight
 
         quantize = self.project_out(quantize)
 
         if need_transpose:
-            quantize = rearrange(quantize, 'b d n -> b n d')
+            quantize = rearrange(quantize, 'b n d -> b d n')
+
+        if self.accept_image_fmap:
+            quantize = rearrange(quantize, 'b (h w) c -> b c h w', h = height, w = width)
+            embed_ind = rearrange(embed_ind, 'b (h w) -> b h w', h = height, w = width)
 
         return quantize, embed_ind, loss
